import json
from typing import List, Union, Dict, Any

from csp_cbc_auth.cbc.session import CBCSession
from psc_auth_integration.entities.auth_user_session import AuthUserSession

from cds_configuration_service.api_clients.base_api_class import ConfigurationServiceBaseClass


class CustomConfigurationServiceV1Api(ConfigurationServiceBaseClass):
    """
    Custom Configuration Service V1 API wrapper
    """

    def __init__(self, session: Union[AuthUserSession, CBCSession]):
        super().__init__(session)
        self.custom_base_url = f'{self.configuration_service_base_url}/pacing_config/v1'

    def get_profile(self, org_key: str, profile_key: str):
        """
        Retrieve a specific profile.

        :param org_key: A string that uniquely identifies the organization.
            Example structure of the `org_key`: "TEST_ORG"
        :param profile_key: A string that uniquely identifies the custom profile to be updated.
            It contains 4 parts: org_key, group_id, date or date range, and content_identifiers.
            Example structure of the `profile_key`: "TEST_ORG:TEST_GROUP_ID:20230714:TEST"
        """
        url = f'{self.custom_base_url}/orgs/{org_key}/profiles/{profile_key}'
        response = self.get(url)
        return response

    def get_all_profiles(self, org_key: str):
        """
        Retrieve all profiles for a specific organization.

        :param org_key: A string that uniquely identifies the organization.
            Example structure of the `org_key`: "TEST_ORG"
        """
        url = f'{self.custom_base_url}/orgs/{org_key}/profiles'
        response = self.get(url)
        return response

    def add_profile(self, org_key: str, profile_data: Dict[str, Any]):
        """
        Add a new profile.

        :param org_key: A string that uniquely identifies the organization.
            Example structure of the `org_key`: "TEST_ORG"
        :param profile_data: A dictionary of profile data.
            Example structure of a single-date `profile_data`:
            {
                  "group_id": "TEST_GROUP_ID",
                  "profile_date": "20230714",
                  "content_identifiers": [
                    "TEST"
                  ],
                  "budget": [
        [50.0,50.0,1.0,1.0,1.0,1.0,1.0,-1.0,1.0,0.0,0.0,0.0,1.0,100.0,100.0,10.0,1.0,100.0,100.0,1.0,1.0,10.0,1.0,1.0],
        [50.0,50.0,1.0,1.0,1.0,1.0,1.0,-1.0,1.0,0.0,0.0,0.0,1.0,100.0,100.0,100.0,1.0,100.0,100.0,1.0,1.0,10.0,1.0,1.0],
        [50.0,50.0,1.0,1.0,1.0,1.0,1.0,-1.0,1.0,0.0,0.0,0.0,1.0,100.0,100.0,100.0,1.0,100.0,100.0,1.0,1.0,10.0,1.0,1.0],
        [50.0,50.0,1.0,1.0,1.0,1.0,1.0,-1.0,1.0,0.0,0.0,0.0,1.0,100.0,100.0,10.0,1.0,10.0,100.0,1.0,1.0,10.0,1.0,1.0],
        [50.0,50.0,1.0,1.0,1.0,1.0,1.0,-1.0,1.0,0.0,0.0,0.0,1.0,100.0,100.0,100.0,1.0,100.0,100.0,1.0,1.0,10.0,1.0,1.0],
        [50.0,50.0,1.0,1.0,1.0,1.0,1.0,-1.0,1.0,0.0,0.0,0.0,1.0,100.0,100.0,100.0,1.0,100.0,100.0,1.0,1.0,10.0,1.0,1.0],
        [50.0,50.0,1.0,1.0,1.0,1.0,1.0,-1.0,1.0,0.0,0.0,0.0,1.0,100.0,100.0,100.0,1.0,100.0,100.0,1.0,1.0,10.0,1.0,1.0]]
            }
            If it's a date-range profile, the `profile_data` will contain a field `profile_date_range`
            instead of `profile_date`.
            {
                ...
                "profile_date_range": {
                "date_from":"20230714",
                "date_to": "20230814"
                }
                ...
            }
        """
        url = f'{self.custom_base_url}/orgs/{org_key}/profiles'
        response = self.post(url, json=profile_data)
        return response

    def update_profile(self, org_key: str, profile_key: str, budget_data: List[List[float]]):
        """
        Update an existing custom profile's budget.

        :param org_key: A string that uniquely identifies the organization.
            Example structure of the `org_key`: "TEST_ORG"
        :param profile_key: A string that uniquely identifies the custom profile to be updated.
            It contains 4 parts: org_key, group_id, date or date range, and content_identifiers.
            Example structure of the `profile_key`: "TEST_ORG:TEST_GROUP_ID:20230714:TEST"
        :param budget_data: A list of lists containing float values that represent the budget allocations.
            It should be a 7x24 matrix where each inner list contains 24 float values for each hour of the day,
            and the outer list contains 7 of these inner lists, representing a week. Each float value represents
            the specific budget setting for that hour.

            Example structure of the `budget_data`:
            [
                [100.0, 50.0, 1.0, ..., 1.0, 1.0],
                [50.0, 50.0, 1.0, ..., 1.0, 1.0],
                ...
                [50.0, 50.0, 1.0, ..., 1.0, 1.0]
            ]
        """
        url = f'{self.custom_base_url}/orgs/{org_key}/profiles/{profile_key}'
        data = json.dumps(budget_data)
        response = self.patch(url, data=data)
        return response

    def delete_profile(self, org_key: str, profile_key: str):
        """
        Delete a specific profile.

        :param org_key: A string that uniquely identifies the organization.
            Example structure of the `org_key`: "TEST_ORG"
        :param profile_key: A string that uniquely identifies the custom profile to be deleted.
            It contains 4 parts: org_key, group_id, date or date range, and content_identifiers.
            Example structure of the `profile_key`: "TEST_ORG:TEST_GROUP_ID:20230714:TEST"
        """
        url = f'{self.custom_base_url}/orgs/{org_key}/profiles/{profile_key}'
        response = self.delete(url)
        return response
